/* $USAGI: rawsend.c,v 1.3 2004/01/08 16:19:06 yoshfuji Exp $ */

/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <getopt.h>

int main(int argc, char **argv) {
	char *host, *serv, *proto = "ipv6-icmp";
	struct addrinfo hints, *res;
#if !HAVE_GETADDRINFO_FOR_RAW
	struct protoent *pe;
#endif
	int gai;
	int s;
	int len;
	char buf[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	int off, i, c;
	int min = -1, max = -1;
	int usage = 0;
	char *myname = argv[0];

	while ((c = getopt(argc, argv, "m:M:p:")) != EOF) {
		switch(c) {
		case 'm':
			min = atoi(optarg);
			if (min > (int)sizeof(buf))
				usage = 1;
			break;
		case 'M':
			max = atoi(optarg);
			if (max > (int)sizeof(buf))
				usage = 1;
			break;
		case 'p':
			proto = optarg;
			break;
		}
	}

	argc -= optind;
	argv += optind;

	if (usage || !argc) {
		fprintf(stderr, "Usage: %s [-m min] [-M max] [-p proto] host\n", myname);
		exit(1);
	}
	printf("%s\n", argv);

	host = argv[0];

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_UNSPEC;
	hints.ai_socktype = SOCK_RAW;
#if HAVE_GETADDRINFO_FOR_RAW
	hints.ai_protocol = 0;
#else
	pe = getprotobyname(proto);
	if (!pe) {
		perror("getprotobyname");
		exit(1);
	}
	hints.ai_protocol = pe->p_proto;
#endif
#if HAVE_GETADDRINFO_FOR_RAW
	gai = getaddrinfo(host, proto, &hints, &res);
#else
	gai = getaddrinfo(host, NULL, &hints, &res);
#endif
	if (gai) {
		printf("getaddrinfo(): %s\n",
			gai_strerror(gai));
		exit(1);
	}

	s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	if (s < 0) {
		perror("socket");
		exit(1);
	}

	for (i = min; i <= max; i++){
		if (i == 0 || i == 1)
			continue;
		for (off = 0; off < 8; off++)
			buf[off]=off;
		off = i;
		if (setsockopt(s, IPPROTO_RAW, IPV6_CHECKSUM, &i, sizeof(i)) < 0) {
			perror("setsockopt(CHECKSUM)");
			//exit(1);
		}

		if ((len = sendto(s, buf, sizeof(buf), 0,
					res->ai_addr, res->ai_addrlen)) < 0) {
			perror("sendto");
			close(s);
			exit(1);
		}
		printf ("i = %d, len = %d, sent = %d\n", i, sizeof(buf), len);
	}
	close(s);
	exit(0);
}
